@@ -6,11 +6,11 @@ from math import log2
 from typing import Callable, Iterable, Literal, Sequence, cast, overload
 
 from vsaa import Eedi3, Nnedi3, SuperSampler
-from vskernels import Catrom, Kernel, KernelT, Scaler, Spline144, ScalerT
-from vsmask.edge import EdgeDetect
+from vskernels import Catrom, Kernel, KernelT, Scaler, ScalerT, Spline144
+from vsmasktools import EdgeDetect, EdgeDetectT, Prewitt
 from vstools import (
-    check_variable, core, depth, get_depth, get_h, get_prop, get_w, get_y, join, normalize_seq, split, vs,
-    CustomValueError
+    CustomValueError, FieldBased, FieldBasedT, FuncExceptT, check_variable, core, depth, get_depth, get_h, get_prop,
+    get_w, get_y, join, normalize_seq, split, vs
 )
 
 from .helpers import scale_var_clip
@@ -21,7 +21,9 @@ from .types import CreditMaskT, DescaleAttempt, DescaleMode, DescaleResult, Plan
 __all__ = [
     'get_select_descale', 'descale',
 
-    'mixed_rescale'
+    'mixed_rescale',
+
+    'descale_fields'
 ]
 
 
@@ -136,18 +138,15 @@ def get_select_descale(
     return _select_descale, diff_clips
 
 
-DescKernelsT = KernelT | Sequence[KernelT]
-
-
 @overload
 def descale(  # type: ignore
     clip: vs.VideoNode,
     width: int | Iterable[int] | None = None, height: int | Iterable[int] = 720,
-    kernels: DescKernelsT = Catrom(), upscaler: Scaler | None = Nnedi3(),
+    kernels: KernelT | Sequence[KernelT] = Catrom, upscaler: ScalerT | None = Nnedi3,
     mask: CreditMaskT | Literal[False] = descale_detail_mask,
     mode: DescaleMode = DescaleMode.PlaneDiff(0.0),
     dst_width: int | None = None, dst_height: int | None = None,
-    shift: tuple[float, float] = (0, 0), scaler: Kernel = Spline144(),
+    shift: tuple[float, float] = (0, 0), scaler: ScalerT = Spline144,
     result: Literal[False] = False
 ) -> vs.VideoNode:
     ...
@@ -157,11 +156,11 @@ def descale(  # type: ignore
 def descale(  # type: ignore
     clip: vs.VideoNode,
     width: int | Iterable[int] | None = None, height: int | Iterable[int] = 720,
-    kernels: DescKernelsT = Catrom(), upscaler: Scaler = Nnedi3(),
+    kernels: KernelT | Sequence[KernelT] = Catrom, upscaler: ScalerT = Nnedi3,
     mask: CreditMaskT = descale_detail_mask,
     mode: DescaleMode = DescaleMode.PlaneDiff(0.0),
     dst_width: int | None = None, dst_height: int | None = None,
-    shift: tuple[float, float] = (0, 0), scaler: Kernel = Spline144(),
+    shift: tuple[float, float] = (0, 0), scaler: ScalerT = Spline144,
     result: Literal[True] = True
 ) -> _DescaleTypeGuards.UpscalerNotNoneMaskNotNone:
     ...
@@ -171,11 +170,11 @@ def descale(  # type: ignore
 def descale(  # type: ignore
     clip: vs.VideoNode,
     width: int | Iterable[int] | None = None, height: int | Iterable[int] = 720,
-    kernels: DescKernelsT = Catrom(), upscaler: Scaler = Nnedi3(),
+    kernels: KernelT | Sequence[KernelT] = Catrom, upscaler: ScalerT = Nnedi3,
     mask: Literal[False] = False,
     mode: DescaleMode = DescaleMode.PlaneDiff(0.0),
     dst_width: int | None = None, dst_height: int | None = None,
-    shift: tuple[float, float] = (0, 0), scaler: Kernel = Spline144(),
+    shift: tuple[float, float] = (0, 0), scaler: ScalerT = Spline144,
     result: Literal[True] = True
 ) -> _DescaleTypeGuards.UpscalerNotNoneMaskIsNone:
     ...
@@ -185,11 +184,11 @@ def descale(  # type: ignore
 def descale(  # type: ignore
     clip: vs.VideoNode,
     width: int | Iterable[int] | None = None, height: int | Iterable[int] = 720,
-    kernels: DescKernelsT = Catrom(), upscaler: None = None,
+    kernels: KernelT | Sequence[KernelT] = Catrom, upscaler: None = None,
     mask: CreditMaskT = descale_detail_mask,
     mode: DescaleMode = DescaleMode.PlaneDiff(0.0),
     dst_width: int | None = None, dst_height: int | None = None,
-    shift: tuple[float, float] = (0, 0), scaler: Kernel = Spline144(),
+    shift: tuple[float, float] = (0, 0), scaler: ScalerT = Spline144,
     result: Literal[True] = True
 ) -> _DescaleTypeGuards.UpscalerIsNoneMaskNotNone:
     ...
@@ -199,11 +198,11 @@ def descale(  # type: ignore
 def descale(
     clip: vs.VideoNode,
     width: int | Iterable[int] | None = None, height: int | Iterable[int] = 720,
-    kernels: DescKernelsT = Catrom(), upscaler: None = None,
+    kernels: KernelT | Sequence[KernelT] = Catrom, upscaler: None = None,
     mask: Literal[False] = False,
     mode: DescaleMode = DescaleMode.PlaneDiff(0.0),
     dst_width: int | None = None, dst_height: int | None = None,
-    shift: tuple[float, float] = (0, 0), scaler: Kernel = Spline144(),
+    shift: tuple[float, float] = (0, 0), scaler: ScalerT = Spline144,
     result: Literal[True] = True
 ) -> _DescaleTypeGuards.UpscalerIsNoneMaskIsNone:
     ...
@@ -213,11 +212,11 @@ def descale(
 def descale(
     clip: vs.VideoNode,
     width: int | Iterable[int] | None = None, height: int | Iterable[int] = 720,
-    kernels: DescKernelsT = Catrom(), upscaler: Scaler | None = Nnedi3(),
+    kernels: KernelT | Sequence[KernelT] = Catrom, upscaler: ScalerT | None = Nnedi3,
     mask: CreditMaskT | Literal[False] = descale_detail_mask,
     mode: DescaleMode = DescaleMode.PlaneDiff(0.0),
     dst_width: int | None = None, dst_height: int | None = None,
-    shift: tuple[float, float] = (0, 0), scaler: Kernel = Spline144(),
+    shift: tuple[float, float] = (0, 0), scaler: ScalerT = Spline144,
     result: Literal[True] = True
 ) -> DescaleResult:
     ...
@@ -226,11 +225,11 @@ def descale(
 def descale(
     clip: vs.VideoNode,
     width: int | Iterable[int] | None = None, height: int | Iterable[int] = 720,
-    kernels: DescKernelsT = Catrom(), upscaler: Scaler | None = Nnedi3(),
+    kernels: KernelT | Sequence[KernelT] = Catrom, upscaler: ScalerT | None = Nnedi3,
     mask: CreditMaskT | Literal[False] = descale_detail_mask,
     mode: DescaleMode = DescaleMode.PlaneDiff(0.0),
     dst_width: int | None = None, dst_height: int | None = None,
-    shift: tuple[float, float] = (0, 0), scaler: Kernel = Spline144(),
+    shift: tuple[float, float] = (0, 0), scaler: ScalerT = Spline144,
     result: bool = False
 ) -> vs.VideoNode | DescaleResult:
     assert clip.format
@@ -258,10 +257,9 @@ def descale(
         kernels = [kernels]
 
     norm_resolutions = list(zip(widths, heights))
-    norm_kernels = [
-        kernel if isinstance(kernel, Kernel) else Kernel.ensure_obj(kernel, descale)
-        for kernel in kernels
-    ]
+    norm_kernels = [Kernel.ensure_obj(kernel, descale) for kernel in kernels]
+
+    scaler = Scaler.ensure_obj(scaler, descale)
 
     if len(widths) != len(heights):
         raise CustomValueError('Number of heights and widths specified mismatch!', descale)
@@ -318,18 +316,20 @@ def descale(
 
     upscaled = None
     if upscaler:
-        upscaled = scale_var_clip(descaled, dest_width, dest_height, scaler=upscaler)
+        upscaled = scale_var_clip(
+            descaled, dest_width, dest_height, scaler=Scaler.ensure_obj(upscaler, descale)
+        )
         upscaled = depth(upscaled, bit_depth)
 
     if mask:
-        if isinstance(mask, EdgeDetect):
-            mask = mask.edgemask(clip_y)
+        if isinstance(mask, EdgeDetectT):  # type: ignore
+            mask = EdgeDetect.ensure_obj(mask).edgemask(clip_y)  # type: ignore
         elif callable(mask):
-            mask = mask(clip, rescaled)
+            mask = mask(clip, rescaled)  # type: ignore
 
         assert isinstance(mask, vs.VideoNode)
 
-        mask = depth(mask, bit_depth)
+        mask = depth(mask, bit_depth)  # type: ignore
 
         if upscaled:
             mask = scaler.scale(mask, dest_width, dest_height)
@@ -339,9 +339,9 @@ def descale(
     if upscaled:
         out = upscaled
     elif upscaler:
-        out = descaled
+        out = rescaled
     else:
-        out = clip
+        out = descaled
 
     if chroma and upscaled and (clip.width, clip.height) == (dest_width, dest_height):
         out = join([upscaled, *chroma], clip.format.color_family)
@@ -419,11 +419,13 @@ def mixed_rescale(
 
     if credit_mask:
         if isinstance(credit_mask, vs.VideoNode):
-            detail_mask = depth(credit_mask, get_depth(clip))
-        elif callable(credit_mask):
+            detail_mask = depth(credit_mask, get_depth(clip))  # type: ignore
+        elif callable(credit_mask) and not isinstance(credit_mask, type):
             detail_mask = credit_mask(clip_y, upscaled)
-        elif isinstance(credit_mask, EdgeDetect):
-            detail_mask = credit_mask.edgemask(merged)
+        else:
+            detail_mask = EdgeDetect.ensure_obj(
+                Prewitt if credit_mask is True else credit_mask  # type: ignore
+            ).edgemask(merged)
 
         detail_mask = detail_mask.std.Limiter()
     else:
@@ -452,6 +454,70 @@ def mixed_rescale(
     return core.std.ShufflePlanes([masked, clip], planes=[0, 1, 2], colorfamily=vs.YUV)
 
 
+def descale_fields(
+    clip: vs.VideoNode, width: int | None = None, height: int = 720,
+    tff: bool | FieldBasedT = True, kernel: KernelT = Catrom,
+    src_top: float | tuple[float, float] = 0.0,
+    src_left: float | tuple[float, float] = 0.0,
+    debug: bool = False, func: FuncExceptT | None = None
+) -> vs.VideoNode:
+    """
+    Descale interwoven upscaled fields, also known as a cross conversion.
+
+    ``src_top``, ``src_left`` allow you to to shift the clip prior to descaling.
+    This may be useful, as sometimes clips are shifted before or after the original upscaling.
+
+    :param clip:        Clip to process.
+    :param width:       Native width. Will be automatically determined if set to `None`.
+    :param height:      Native height. Will be divided by two internally.
+    :param tff:         Top-field-first. `False` sets it to Bottom-Field-First.
+    :param kernel:      py:class:`vskernels.Kernel` used for the descaling.
+    :param src_top:     Shifts the clip vertically during the descaling.
+                        Can be a tuple, defining the shift per-field.
+    :param src_left:    Shifts the clip horizontally during the descaling.
+                        Can be a tuple, defining the shift per-field.
+    :param debug:       Set a frameprop with the kernel that was used.
+
+    :return:            Descaled GRAY clip.
+    """
+
+    func = func or descale_fields
+
+    height_field = int(height / 2)
+    width = width or get_w(height, clip)
+
+    kernel = Kernel.ensure_obj(kernel, func)
+
+    clip = FieldBased.ensure_presence(clip, tff, func)
+
+    y = get_y(clip).std.SeparateFields()
+
+    if isinstance(src_top, tuple):
+        ff_top, sf_top = src_top
+    else:
+        ff_top = sf_top = src_top
+
+    if isinstance(src_left, tuple):
+        ff_left, sf_left = src_left
+    else:
+        ff_left = sf_left = src_left
+
+    if (ff_top, ff_left) == (sf_top, sf_left):
+        descaled = kernel.descale(y, width, height_field, (ff_top, ff_left))
+    else:
+        descaled = core.std.Interleave([
+            kernel.descale(y[::2], width, height_field, (ff_top, ff_left)),
+            kernel.descale(y[1::2], width, height_field, (sf_top, sf_left))
+        ])
+
+    weave_y = descaled.std.DoubleWeave()
+
+    if debug:
+        weave_y = weave_y.std.SetFrameProp('scaler', data=f'{kernel.__class__.__name__} (Fields)')
+
+    return weave_y.std.SetFieldBased(0)[::2]
+
+
 # TODO: Write a function that checks every possible combination of B and C in bicubic
 #       and returns a list of the results.
 #       Possibly return all the frames in order of smallest difference to biggest.
