@@ -6,11 +6,11 @@ from math import ceil
 from typing import Any
 
 from vsexprtools import aka_expr_available, expr_func
-from vskernels import Bilinear, Catrom, Scaler, ScalerT, SetsuCubic
+from vskernels import Catrom, Scaler, ScalerT, SetsuCubic
 from vsrgtools import box_blur, gauss_blur
 from vstools import (
-    Matrix, MatrixT, PlanesT, Transfer, VSFunction, check_variable, core, depth, fallback, get_depth, get_w,
-    inject_self, vs, CustomValueError, check_ref_clip
+    CustomValueError, Matrix, MatrixT, PlanesT, Transfer, VSFunction, check_ref_clip, check_variable, core, depth,
+    fallback, get_depth, get_w, inject_self, vs
 )
 
 from .gamma import gamma2linear, linear2gamma
@@ -24,21 +24,20 @@ __all__ = [
 
 
 @dataclass
-class DPID(Scaler):
+class DPID(GenericScaler):
     sigma: float = 0.1
     ref: vs.VideoNode | ScalerT | None = None
-    scaler: ScalerT = Bilinear
     planes: PlanesT = None
 
     @inject_self
     def scale(  # type: ignore[override]
         self, clip: vs.VideoNode, width: int, height: int, shift: tuple[float, float] = (0, 0), **kwargs: Any
     ) -> vs.VideoNode:
+        ref = clip
+
         if isinstance(self.ref, vs.VideoNode):
-            check_ref_clip(clip, self.ref)
-            ref = self.ref
-        else:
-            ref = clip
+            check_ref_clip(clip, self.ref)  # type: ignore
+            ref = self.ref  # type: ignore
 
         scaler = Scaler.ensure_obj(self.ref if isinstance(self.ref, Scaler) else self.scaler, self.__class__)
 
@@ -75,7 +74,7 @@ class SSIM(Scaler):
 
 def ssim_downsample(
     clip: vs.VideoNode, width: int | None = None, height: int = 720,
-    smooth: float | VSFunction = ((3 ** 2 - 1) / 12) ** 0.5,
+    smooth: int | float | VSFunction = ((3 ** 2 - 1) / 12) ** 0.5,
     scaler: ScalerT = Catrom,
     curve: Transfer | bool = False, sigmoid: bool = False,
     shift: tuple[float, float] = (0, 0), epsilon: float = 1e-6
@@ -170,11 +169,6 @@ class DLISR(GenericScaler):
     matrix: MatrixT | None = None
     device_id: int | None = None
 
-    def __post_init__(self) -> None:
-        super().__post_init__()
-
-        self._scaler = Scaler.ensure_obj(self.scaler, self.__class__)
-
     @inject_self
     def scale(  # type: ignore
         self, clip: vs.VideoNode, width: int, height: int, shift: tuple[float, float] = (0, 0),
@@ -195,14 +189,4 @@ class DLISR(GenericScaler):
 
             output = output.akarin.DLISR(max_scale, self.device_id)
 
-        if (clip.width, clip.height) != (width, height):
-            output = self._scaler.scale(output, width, height, shift)
-        elif shift != (0, 0):
-            output = self._kernel.shift(output, shift)
-
-        assert check_variable(output, self.__class__)
-
-        if output.format.id == clip.format.id:
-            return output
-
-        return self._kernel.resample(output, clip, matrix)
+        return self._finish_scale(output, clip, width, height, shift, matrix)
