@@ -1,15 +1,13 @@
 from __future__ import annotations
 
 from dataclasses import dataclass, field
-from functools import cached_property
 from math import ceil
 from pathlib import Path
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, overload
 
-from vskernels import Catrom, Kernel, Scaler, ScalerT, KernelT
 from vstools import (
-    CustomRuntimeError, FileWasNotFoundError, core, expect_bits, get_user_data_dir, get_video_format, get_y,
-    inject_self, join, vs, CustomNotImplementedError
+    MISSING, CustomRuntimeError, FileWasNotFoundError, MissingT, core, expect_bits, get_user_data_dir, get_video_format,
+    inject_self, join, vs
 )
 
 from .base import ShaderFileBase, ShaderFileCustom
@@ -47,17 +45,9 @@ class PlaceboShaderBase(PlaceboShaderMeta):
     param1: float | None = field(default=None, kw_only=True)
     param2: float | None = field(default=None, kw_only=True)
 
-    scaler: ScalerT = field(default=Catrom, kw_only=True)
-    shifter: KernelT | None = field(default=None, kw_only=True)
-
     def __post_init__(self) -> None:
         super().__post_init__()
 
-        self._scaler = Scaler.ensure_obj(self.scaler, self.__class__)
-        self._shifter = Kernel.ensure_obj(
-            self.shifter or (self._scaler if isinstance(self._scaler, Kernel) else Catrom), self.__class__
-        )
-
         if not hasattr(self, 'shader_file'):
             raise CustomRuntimeError('You must specify a "shader_file"!', self.__class__)
 
@@ -65,25 +55,29 @@ class PlaceboShaderBase(PlaceboShaderMeta):
     def scale(  # type: ignore
         self, clip: vs.VideoNode, width: int, height: int, shift: tuple[float, float] = (0, 0), **kwargs: Any
     ) -> vs.VideoNode:
-        clip, _ = expect_bits(clip, 16)
+        output, _ = expect_bits(clip, 16)
 
-        fmt = get_video_format(clip)
+        fmt = get_video_format(output)
 
         if fmt.num_planes == 1:
-            if width > clip.width or height > clip.height:
-                clip = clip.resize.Point(format=vs.YUV444P16)
+            if width > output.width or height > output.height:
+                output = output.resize.Point(format=vs.YUV444P16)
             else:
                 for div in (4, 2):
                     if width % div == 0 and height % div == 0:
-                        blank = core.std.BlankClip(clip, clip.width // div, clip.height // div, vs.GRAY16)
+                        blank = core.std.BlankClip(output, output.width // div, output.height // div, vs.GRAY16)
                         break
                 else:
-                    blank = clip.std.BlankClip(vs.GRAY16)
+                    blank = output.std.BlankClip(vs.GRAY16)
 
-                clip = join(clip, blank, blank)
+                output = join(output, blank, blank)
 
         kwargs |= {
-            'shader': self.shader_file.path if isinstance(self.shader_file, ShaderFile) else str(self.shader_file),
+            'shader': str(
+                self.shader_file()
+                if isinstance(self.shader_file, ShaderFile) else
+                ShaderFile.CUSTOM(self.shader_file)
+            ),
             'chroma_loc': self.chroma_loc, 'matrix': self.matrix,
             'trc': self.trc, 'linearize': self.linearize,
             'sigmoidize': self.sigmoidize, 'sigmoid_center': self.sigmoid_center, 'sigmoid_slope': self.sigmoid_slope,
@@ -92,8 +86,8 @@ class PlaceboShaderBase(PlaceboShaderMeta):
             'blur': self.blur, 'taper': self.taper, 'radius': self.radius,
             'param1': self.param1, 'param2': self.param2,
         } | kwargs | {
-            'width': clip.width * ceil(width / clip.width),
-            'height': clip.height * ceil(height / clip.height)
+            'width': output.width * ceil(width / output.width),
+            'height': output.height * ceil(height / output.height)
         }
 
         if not kwargs['filter']:
@@ -105,21 +99,9 @@ class PlaceboShaderBase(PlaceboShaderMeta):
             except FileWasNotFoundError:
                 ...
 
-        clip = clip.placebo.Shader(**kwargs)
+        output = output.placebo.Shader(**kwargs)
 
-        if fmt.num_planes == 1:
-            clip = get_y(clip)
-
-        if (clip.width, clip.height) != (width, height):
-            clip = self._scaler.scale(clip, width, height)
-
-        if shift != (0, 0):
-            if self.shifter:
-                clip = self._shifter.shift(clip, shift)
-            else:
-                clip = self._kernel.shift(clip, shift)
-
-        return self._kernel.resample(clip, fmt)
+        return self._finish_scale(output, clip, width, height, shift)
 
 
 @dataclass
@@ -138,19 +120,35 @@ class ShaderFile(ShaderFileBase):
     SSIM_DOWNSCALER = 'SSimDownscaler.glsl'
     SSIM_SUPERSAMPLER = 'SSimSuperRes.glsl'
 
+    @overload
+    def __call__(self) -> Path:
+        ...
+
+    @overload
     def __call__(self: ShaderFileCustom, file_name: str | Path) -> Path:  # type: ignore
+        ...
+
+    def __call__(self, file_name: str | Path | MissingT = MISSING) -> Path:
         if self is not ShaderFile.CUSTOM:
-            raise CustomNotImplementedError
+            return Path(__file__).parent / 'shaders' / self.value
 
-        in_cwd = Path.cwd() / file_name
+        if file_name is MISSING:  # type: ignore
+            raise TypeError("ShaderFile.__call__() missing 1 required positional argument: 'file_name'")
 
-        if in_cwd.is_file():
-            return in_cwd
+        file_name, cwd = Path(file_name), Path.cwd()
 
-        asset_dir = Path.cwd() / '_assets' / file_name
+        assets_dirs = [
+            file_name,
+            cwd / file_name,
+            cwd / '.shaders' / file_name,
+            cwd / '_shaders' / file_name,
+            cwd / '.assets' / file_name,
+            cwd / '_assets' / file_name
+        ]
 
-        if asset_dir.is_file():
-            return asset_dir
+        for asset_dir in assets_dirs:
+            if asset_dir.is_file():
+                return asset_dir
 
         mpv_dir = get_user_data_dir().parent / 'Roaming' / 'mpv' / 'shaders' / file_name
 
@@ -159,13 +157,6 @@ class ShaderFile(ShaderFileBase):
 
         raise FileWasNotFoundError(f'"{file_name}" could not be found!', str(ShaderFile.CUSTOM))
 
-    @cached_property
-    def path(self) -> Path:
-        if self is ShaderFile.CUSTOM:
-            raise CustomNotImplementedError
-
-        return Path(__file__).parent / 'shaders' / self.value
-
 
 class FSRCNNXShader(PlaceboShaderBase):
     shader_file = ShaderFile.FSRCNNX_x56
